@@ -4,7 +4,7 @@
 
 import simplejson
 
-from zope.component import getMultiAdapter, queryMultiAdapter
+from zope.component import getMultiAdapter, getUtility, queryMultiAdapter
 from zope.event import notify
 from zope.interface import Attribute, implements, providedBy
 from zope.interface.interfaces import IInterface
@@ -16,10 +16,12 @@ from zope.security.proxy import isinstance as zope_isinstance
 from lazr.lifecycle.event import ObjectModifiedEvent
 from lazr.lifecycle.snapshot import Snapshot
 
+from lazr.restful.fields import CollectionField
 from lazr.restful.interfaces import (
     ICollection, IFieldMarshaller, IResourceDELETEOperation,
-    IResourceGETOperation, IResourcePOSTOperation)
+    IResourceGETOperation, IResourcePOSTOperation, IWebServiceConfiguration)
 from lazr.restful.interfaces import ICollectionField, IReference
+from lazr.restful.utils import is_total_size_link_active
 from lazr.restful._resource import (
     BatchingResourceMixin, CollectionResource, ResourceJSONEncoder)
 
@@ -44,6 +46,28 @@ class ResourceOperation(BatchingResourceMixin):
     def __init__(self, context, request):
         self.context = context
         self.request = request
+        self.total_size_only = False
+
+    def total_size_link(self, navigator):
+        """Return a link to the total size of a collection."""
+        # If the version we're being asked for is equal to or later
+        # than the version in which we started exposing
+        # total_size_link, then include it; otherwise include
+        # total_size.
+        config = getUtility(IWebServiceConfiguration)
+        if not is_total_size_link_active(self.request.version, config):
+            # This is a named operation that includes the total size
+            # inline rather than with a link.
+            return None
+        if not IResourceGETOperation.providedBy(self):
+            # Only GET operations can have their total size split out into
+            # a link, because only GET operations are safe.
+            return None
+        base = str(self.request.URL)
+        query = navigator.getCleanQueryString()
+        if query != '':
+            query += '&'
+        return base + '?' + query + "ws.show=total_size"
 
     def __call__(self):
         values, errors = self.validate()
@@ -80,13 +104,22 @@ class ResourceOperation(BatchingResourceMixin):
             # this object served to the client.
             return result
 
+        # The similar patterns in the two branches below suggest some deeper
+        # symmetry that should be extracted.
         if queryMultiAdapter((result, self.request), ICollection):
             # If the result is a web service collection, serve only one
             # batch of the collection.
             collection = getMultiAdapter((result, self.request), ICollection)
-            result = CollectionResource(collection, self.request).batch() + '}'
+            resource = CollectionResource(collection, self.request)
+            if self.total_size_only:
+                result = resource.get_total_size(collection)
+            else:
+                result = resource.batch() + '}'
         elif self.should_batch(result):
-            result = self.batch(result, self.request) + '}'
+            if self.total_size_only:
+                result = self.get_total_size(result)
+            else:
+                result = self.batch(result, self.request) + '}'
         else:
             # Serialize the result to JSON. Any embedded entries will be
             # automatically serialized.
@@ -158,14 +191,14 @@ class ResourceOperation(BatchingResourceMixin):
                     value = marshaller.marshall_from_request(
                         self.request.form.get(name))
                 except ValueError, e:
-                    errors.append("%s: %s" % (name, e))
+                    errors.append(u"%s: %s" % (name, e))
                     continue
             try:
                 field.validate(value)
             except RequiredMissing:
-                errors.append("%s: Required input is missing." % name)
+                errors.append(u"%s: Required input is missing." % name)
             except ValidationError, e:
-                errors.append("%s: %s" % (name, e))
+                errors.append(u"%s: %s" % (name, e))
             else:
                 validated_values[name] = value
         return (validated_values, errors)
