@@ -35,6 +35,7 @@ class BelowRootAbsoluteURL(AbsoluteURL):
 class WebServiceConfiguration(BaseWSGIWebServiceConfiguration):
     code_revision = '1'
     active_versions = ['beta', '1.0', '2.0', '3.0', 'trunk']
+    first_version_with_total_size_link = '2.0'
     last_version_with_mutator_named_operations = '1.0'
     use_https = False
     view_permission = 'zope.Public'
@@ -46,6 +47,8 @@ class MultiversionWebServiceRootResource(RootResource):
         pairset = PairSet()
         pairset.pairs = [
             KeyValuePair(pairset, "foo", "bar"),
+            KeyValuePair(pairset, "foo2", "bar"),
+            KeyValuePair(pairset, "foo3", "bar"),
             KeyValuePair(pairset, "1", "2"),
             KeyValuePair(pairset, "Some", None),
             KeyValuePair(pairset, "Delete", "me"),
