@@ -32,6 +32,7 @@ __all__ = [
     'IFieldMarshaller',
     'IFileLibrarian',
     'IHTTPResource',
+    'INotificationsProvider',
     'IJSONPublishable',
     'IJSONRequestCache',
     'IRepresentationCache',
@@ -48,20 +49,35 @@ __all__ = [
     'IWebBrowserInitiatedRequest',
     'LAZR_WEBSERVICE_NAME',
     'LAZR_WEBSERVICE_NS',
+    'IWebBrowserOriginatingRequest',
     'IWebServiceClientRequest',
+    'IWebServiceExceptionView',
     'IWebServiceLayer',
     'IWebServiceVersion',
     ]
 
-from zope.schema import Bool, Dict, Int, List, Text, TextLine
-from zope.interface import Attribute, Interface
+from textwrap import dedent
+from zope.schema import (
+    Bool,
+    Dict,
+    Int,
+    List,
+    Text,
+    TextLine,
+    )
+from zope.interface import (
+    Attribute,
+    Interface,
+    )
 # These two should really be imported from zope.interface, but
 # the import fascist complains because they are not in __all__ there.
 from zope.interface.interface import invariant
 from zope.interface.exceptions import Invalid
 from zope.publisher.interfaces import IPublishTraverse
 from zope.publisher.interfaces.browser import (
-    IBrowserRequest, IDefaultBrowserLayer)
+    IBrowserRequest,
+    IDefaultBrowserLayer,
+    )
 from lazr.batchnavigator.interfaces import InvalidBatchSizeError
 
 # Constants for periods of time
@@ -88,11 +104,15 @@ class IHTTPResource(Interface):
 class IJSONPublishable(Interface):
     """An object that can be published as a JSON data structure."""
 
-    def toDataForJSON():
+    def toDataForJSON(media_type):
         """Return a representation that can be turned into JSON.
 
         The representation must consist entirely of simple data
         structures and IJSONPublishable objects.
+
+        :param media_type: The media type that the data will be
+        converted to. This will be application/json, obviously, but it
+        may include parameters.
         """
 
 class IServiceRootResource(IHTTPResource):
@@ -230,6 +250,12 @@ class IFieldHTMLRenderer(Interface):
         """Render the given string as HTML."""
 
 
+class IWebServiceExceptionView(Interface):
+    """The view that defines how errors are related to API clients."""
+
+    status = Attribute("The HTTP status code to return.")
+
+
 class IEntryField(Interface):
     """An individual field of an entry."""
 
@@ -255,6 +281,32 @@ class ITopLevelEntryLink(Interface):
                            "other end of the link.")
 
 
+class IWebBrowserOriginatingRequest(Interface):
+    """A browser request to an object also published on the web service.
+
+    A web framework may define an adapter for this interface, allowing
+    the web service to include, as part of an entry's representation,
+    a link to that same entry as found on the website.
+    """
+
+
+class INotificationsProvider(Interface):
+    """A response object which contains notifications.
+
+    A web framework may define an adapter for this interface, allowing
+    the web service to provide notifications to be sent to the client.
+    """
+    notifications = Attribute(dedent("""\
+        A list of namedtuples (level, message) which can be used by the
+        caller to display extra information about the completed request.
+        'level' matches the standard logging levels:
+
+        DEBUG = logging.DEBUG     # a debugging message
+        INFO = logging.INFO       # simple confirmation of a change
+        WARNING = logging.WARNING # action will not be successful unless...
+        ERROR = logging.ERROR     # the previous action did not succeed
+        """))
+
 class IWebServiceClientRequest(IBrowserRequest):
     """Interface for requests to the web service."""
     version = Attribute("The version of the web service that the client "
@@ -495,6 +547,13 @@ class IWebServiceConfiguration(Interface):
         default = {}
         )
 
+    require_explicit_versions = Bool(
+        default=False,
+        description=u"""If true, each exported field and named
+        operation must explicitly declare the first version in which
+        it appears. If false, fields and operations are published in
+        all versions.""")
+
     last_version_with_mutator_named_operations = TextLine(
         default=None,
         description=u"""In earlier versions of lazr.restful, mutator methods
@@ -507,6 +566,13 @@ class IWebServiceConfiguration(Interface):
         all subsequent versions, they will not be published as named
         operations.""")
 
+    first_version_with_total_size_link = TextLine(
+        default=None,
+        description=u"""In earlier versions of lazr.restful collections
+        included a total_size field, now they include a total_size_link
+        instead. Setting this value determines in which version the new
+        behavior takes effect.""")
+
     code_revision = TextLine(
         default=u"",
         description=u"""A string designating the current revision
