@@ -28,17 +28,30 @@ import urllib
 
 import simplejson
 
-from zope.datetime import DateTimeError, DateTimeParser
-from zope.component import getMultiAdapter, getUtility
+from zope.datetime import (
+    DateTimeError,
+    DateTimeParser,
+    )
+from zope.component import (
+    getMultiAdapter,
+    getUtility,
+    )
 from zope.interface import implements
 from zope.publisher.interfaces import NotFound
 from zope.security.proxy import removeSecurityProxy
 from zope.traversing.browser import absoluteURL
 
-from lazr.uri import URI, InvalidURIError
+from lazr.uri import (
+    URI,
+    InvalidURIError,
+    )
 
 from lazr.restful.interfaces import (
-    IFieldMarshaller, IUnmarshallingDoesntNeedValue, IWebServiceConfiguration)
+    IFieldMarshaller,
+    IUnmarshallingDoesntNeedValue,
+    IServiceRootResource,
+    IWebServiceConfiguration,
+    )
 from lazr.restful.utils import safe_hasattr
 
 
@@ -72,9 +85,16 @@ class URLDereferencingMixin:
             request_port = default_port
 
         if not isinstance(url, basestring):
-            raise ValueError("got '%s', expected string: %r" % (
+            raise ValueError(u"got '%s', expected string: %r" % (
                 type(url).__name__, url))
-        uri = URI(url)
+        if url.startswith('/'):
+            # It's a relative URI. Resolve it relative to the root of this
+            # version of the web service.
+            service_root = getUtility(IServiceRootResource)
+            root_uri = absoluteURL(service_root, self.request)
+            uri = URI(root_uri).append(url[1:])
+        else:
+            uri = URI(url)
         protocol = uri.scheme
         host = uri.host
         port = uri.port or default_port
@@ -171,7 +191,7 @@ class SimpleFieldMarshaller:
                 else:
                     expected_name = self._type.__name__
                 raise ValueError(
-                    "got '%s', expected %s: %r" % (
+                    u"got '%s', expected %s: %r" % (
                         type(value).__name__, expected_name, value))
         return value
 
@@ -329,7 +349,7 @@ class TokenizedVocabularyFieldMarshaller(FixedVocabularyFieldMarshaller):
         try:
             return self.field.vocabulary.getTermByToken(str(value)).value
         except LookupError:
-            raise ValueError("%r isn't a valid token" % value)
+            raise ValueError(u"%r isn't a valid token" % value)
 
 
 class DateTimeFieldMarshaller(SimpleFieldMarshaller):
@@ -491,8 +511,8 @@ class SimpleVocabularyLookupFieldMarshaller(FixedVocabularyFieldMarshaller):
                 return item
             valid_titles.append(item.title)
         raise ValueError(
-            ('Invalid value "%s". Acceptable values are: %s' %
-             (value, ', '.join(valid_titles))).encode("utf-8"))
+            u'Invalid value "%s". Acceptable values are: %s' %
+             (value, ', '.join(valid_titles)))
 
     def unmarshall(self, entry, value):
         if value is None:
@@ -505,7 +525,7 @@ class ObjectLookupFieldMarshaller(SimpleFieldMarshaller,
     """A marshaller that turns URLs into data model objects.
 
     This marshaller can be used with a IChoice field (initialized
-    with a vocabulary) or with an IObject field (no vocabulary).
+    with a vocabulary) or with an IReference field (no vocabulary).
     """
 
     def __init__(self, field, request, vocabulary=None):
@@ -539,9 +559,9 @@ class ObjectLookupFieldMarshaller(SimpleFieldMarshaller,
             resource = self.dereference_url(value)
         except NotFound:
             # The URL doesn't correspond to any real object.
-            raise ValueError('No such object "%s".' % value)
+            raise ValueError(u'No such object "%s".' % value)
         except InvalidURIError:
-            raise ValueError('"%s" is not a valid URI.' % value)
+            raise ValueError(u'"%s" is not a valid URI.' % value)
         # We looked up the URL and got the thing at the other end of
         # the URL: a resource. But internally, a resource isn't a
         # valid value for any schema field. Instead we want the object
