@@ -9,33 +9,56 @@ with the Zope publisher.
 __metaclass__ = type
 __all__ = [
     'browser_request_to_web_service_request',
-    'TraverseWithGet',
     'WebServicePublicationMixin',
     'WebServiceRequestTraversal',
     ]
 
 
-import traceback
+import simplejson
 import urllib
+import urlparse
 
 from zope.component import (
-    adapter, getMultiAdapter, getUtility, queryAdapter, queryMultiAdapter)
+    adapter,
+    getMultiAdapter,
+    getUtility,
+    queryAdapter,
+    queryMultiAdapter,
+    )
 from zope.component.interfaces import ComponentLookupError
-from zope.interface import alsoProvides, implementer, implements
+from zope.interface import (
+    alsoProvides,
+    implementer,
+    implements,
+    )
 from zope.publisher.interfaces import NotFound
 from zope.publisher.interfaces.browser import IBrowserRequest
-from zope.schema.interfaces import IBytes, IObject
+from zope.schema.interfaces import IBytes
 from zope.security.checker import ProxyFactory
 
 from lazr.uri import URI
 
 from lazr.restful import (
-    CollectionResource, EntryField, EntryFieldResource,
-    EntryResource, ScopedCollection, ServiceRootResource)
+    CollectionResource,
+    EntryField,
+    EntryFieldResource,
+    EntryResource,
+    ScopedCollection,
+    )
 from lazr.restful.interfaces import (
-    IByteStorage, ICollection, ICollectionField, IEntry, IEntryField,
-    IHTTPResource, IServiceRootResource, IWebBrowserInitiatedRequest,
-    IWebServiceClientRequest, IWebServiceConfiguration, IWebServiceVersion)
+    IByteStorage,
+    ICollection,
+    ICollectionField,
+    IEntry,
+    IEntryField,
+    IHTTPResource,
+    INotificationsProvider,
+    IReference,
+    IServiceRootResource,
+    IWebBrowserInitiatedRequest,
+    IWebServiceClientRequest,
+    IWebServiceConfiguration,
+    )
 from lazr.restful.utils import tag_request_with_version_name
 
 
@@ -82,7 +105,7 @@ class WebServicePublicationMixin:
                 elif IBytes.providedBy(field):
                     return self._traverseToByteStorage(
                         request, entry, field, name)
-                elif IObject.providedBy(field):
+                elif IReference.providedBy(field):
                     sub_entry = getattr(entry, name, None)
                     if sub_entry is None:
                         raise NotFound(ob, name, request)
@@ -165,10 +188,28 @@ class WebServicePublicationMixin:
         # Wrap the resource in a security proxy.
         return ProxyFactory(resource)
 
+    def _processNotifications(self, request):
+        """Add any notification messages to the response headers.
+
+        If the webservice has defined an INotificationsProvider adaptor, use
+        it to include with the response the relevant notification messages
+        and their severity levels.
+        """
+        notifications_provider = INotificationsProvider(request, None)
+        notifications = []
+        if (notifications_provider is not None
+            and notifications_provider.notifications):
+            notifications = ([(notification.level, notification.message)
+                 for notification in notifications_provider.notifications])
+        json_notifications = simplejson.dumps(notifications)
+        request.response.setHeader(
+            'X-Lazr-Notifications', json_notifications)
+
     def callObject(self, request, object):
         """Help web browsers handle redirects correctly."""
         value = super(
             WebServicePublicationMixin, self).callObject(request, object)
+        self._processNotifications(request)
         if request.response.getStatus() / 100 == 3:
             vhost = URI(request.getApplicationURL()).host
             if IWebBrowserInitiatedRequest.providedBy(request):
@@ -186,14 +227,21 @@ class WebServicePublicationMixin:
                 # able to serve the right document.
                 location = request.response.getHeader("Location", None)
                 if location is not None:
-                    accept = request.response.getHeader(
-                        "Accept", "application/json")
+                    accept = request.getHeader("Accept", "application/json")
                     qs_append = "ws.accept=" + urllib.quote(accept)
-                    uri = URI(location)
-                    if uri.query is None:
-                        uri.query = qs_append
+                    # We don't use the URI class because it will raise
+                    # an exception if the Location contains invalid
+                    # characters. Invalid characters may indeed be a
+                    # problem, but let the problem be handled
+                    # somewhere else.
+                    (scheme, netloc, path, query, fragment) = (
+                        urlparse.urlsplit(location))
+                    if query == '':
+                       query = qs_append
                     else:
-                        uri.query += '&' + qs_append
+                       query += '&' + qs_append
+                    uri = urlparse.urlunsplit(
+                        (scheme, netloc, path, query, fragment))
                     request.response.setHeader("Location", str(uri))
         return value
 
@@ -296,7 +344,7 @@ def browser_request_to_web_service_request(
     if web_service_version is None:
         web_service_version = config.active_versions[-1]
 
-    body = website_request.bodyStream.getCacheStream().read()
+    body = website_request.bodyStream.getCacheStream()
     environ = dict(website_request.environment)
     # Zope picks up on SERVER_URL when setting the _app_server attribute
     # of the new request.
