@@ -12,6 +12,7 @@ __all__ = [
     'Request',
     'RootResource',
     'RootResourceAbsoluteURL',
+    'SimulatedWebsiteRequest',
     'TraverseWithGet',
     ]
 
@@ -36,8 +37,14 @@ import grokcore.component
 from lazr.restful import (
     EntryAdapterUtility, HTTPResource, ServiceRootResource)
 from lazr.restful.interfaces import (
-    IRepresentationCache, IServiceRootResource, ITopLevelEntryLink,
-    ITraverseWithGet, IWebServiceConfiguration, IWebServiceLayer)
+    IRepresentationCache,
+    IServiceRootResource,
+    ITopLevelEntryLink,
+    ITraverseWithGet,
+    IWebBrowserOriginatingRequest,
+    IWebServiceConfiguration,
+    IWebServiceLayer,
+    )
 from lazr.restful.publisher import (
     browser_request_to_web_service_request, WebServicePublicationMixin,
     WebServiceRequestTraversal)
@@ -175,6 +182,16 @@ class Request(WebServiceRequestTraversal, BrowserRequest):
     implements(IWebServiceLayer)
 
 
+class SimulatedWebsiteRequest(BrowserRequest):
+    """A (simulated) request to a website as opposed to a web service.
+
+    If you can adapt a web service request to this class (or some
+    other class that implements IWebBrowserOriginatingRequest), you
+    can take advantage of the web_url feature.
+    """
+    implements(IWebBrowserOriginatingRequest)
+
+
 class RootResource(ServiceRootResource, TraverseWithGet):
     """A service root that expects top-level objects to be defined in code.
 
@@ -477,5 +494,6 @@ class DictionaryBasedRepresentationCache(BaseRepresentationCache):
 
 
 BaseWebServiceConfiguration = implement_from_dict(
-    "BaseWebServiceConfiguration", IWebServiceConfiguration, {}, object)
+    "BaseWebServiceConfiguration", IWebServiceConfiguration,
+    {'first_version_with_total_size_link': None}, object)
 
