@@ -6,19 +6,32 @@ __metaclass__ = type
 
 all = ['entry_adapter_for_schema']
 
+import operator
 import simplejson
 import textwrap
 import urllib
 
 from epydoc.markup import DocstringLinker
-from epydoc.markup.restructuredtext import parse_docstring
+from epydoc.markup.restructuredtext import (
+    _DocumentPseudoWriter,
+    _EpydocReader,
+    ParsedRstDocstring,
+    )
+from docutils import io
+from docutils.core import Publisher
 
 from zope.component import (
     adapts, getGlobalSiteManager, getUtility, queryMultiAdapter)
 from zope.interface import implements
 from zope.interface.interfaces import IInterface
-from zope.schema import getFieldsInOrder
-from zope.schema.interfaces import IBytes, IChoice, IDate, IDatetime, IObject
+from zope.schema import getFields
+from zope.schema.interfaces import (
+    IBytes,
+    IChoice,
+    IDate,
+    IDatetime,
+    IObject,
+    )
 from zope.security.proxy import removeSecurityProxy
 from zope.publisher.interfaces.browser import IBrowserRequest
 from zope.traversing.browser import absoluteURL
@@ -28,13 +41,15 @@ from lazr.enum import IEnumeratedType
 from lazr.restful import (
     EntryResource, ResourceJSONEncoder, CollectionResource,
     EntryAdapterUtility, IObjectLink, RESTUtilityBase)
+from lazr.restful._resource import UnknownEntryAdapter
 from lazr.restful.interfaces import (
     ICollection, ICollectionField, IEntry, IJSONRequestCache,
-    IReferenceChoice, IResourceDELETEOperation, IResourceGETOperation,
+    IReference, IResourceDELETEOperation, IResourceGETOperation,
     IResourceOperation, IResourcePOSTOperation, IScopedCollection,
     ITopLevelEntryLink, IWebServiceClientRequest, IWebServiceConfiguration,
     IWebServiceVersion, LAZR_WEBSERVICE_NAME)
-from lazr.restful.utils import get_current_web_service_request
+from lazr.restful.utils import (get_current_web_service_request,
+    is_total_size_link_active)
 
 
 class WadlDocstringLinker(DocstringLinker):
@@ -59,6 +74,49 @@ class WadlDocstringLinker(DocstringLinker):
         return indexterm
 
 
+class _PydocParser:
+    """Encapsulate the state/objects needed to parse docstrings."""
+
+    def __init__(self):
+        # Set up the instance we'll be using to render docstrings.
+        self.errors = []
+        self.writer = _DocumentPseudoWriter()
+        self.publisher = Publisher(_EpydocReader(self.errors),
+            writer=self.writer,
+            source_class=io.StringInput)
+        self.publisher.set_components('standalone', 'restructuredtext',
+            'pseudoxml')
+        settings_overrides={
+            'report_level':10000,
+            'halt_level':10000,
+            'warning_stream':None,
+            }
+        self.publisher.process_programmatic_settings(None,
+            settings_overrides, None)
+        self.publisher.set_destination()
+
+
+    def parse_docstring(self, docstring, errors):
+        """Parse a docstring for eventual transformation into HTML
+
+        This function is a replacement for parse_docstring from
+        epydoc.markup.restructuredtext.parse_docstring.  This function reuses
+        the Publisher instance while the original did not.  Using This
+        function yields significantly faster WADL generation for complex
+        systems.
+        """
+        # Clear any errors from previous calls.
+        del self.errors[:]
+        self.publisher.set_source(docstring, None)
+        self.publisher.publish()
+        # Move any errors into the caller-provided list.
+        errors[:] = self.errors[:]
+        return ParsedRstDocstring(self.writer.document)
+
+
+_PYDOC_PARSER = _PydocParser()
+
+
 WADL_DOC_TEMPLATE = (
     '<wadl:doc xmlns="http://www.w3.org/1999/xhtml">\n%s\n</wadl:doc>')
 
@@ -74,7 +132,7 @@ def generate_wadl_doc(doc):
         return None
     doc = "%s\n%s" % (lines[0], textwrap.dedent("\n".join(lines[1:])))
     errors = []
-    parsed = parse_docstring(doc, errors)
+    parsed = _PYDOC_PARSER.parse_docstring(doc, errors)
     if len(errors) > 0:
         messages = [str(error) for error in errors]
         raise AssertionError(
@@ -120,9 +178,11 @@ class WebLayerAPI:
         if queryMultiAdapter((self.context, request), IEntry):
             resource = EntryResource(self.context, request)
         else:
-            # Just dump it as JSON.
+            # Just dump it as JSON+XHTML
             resource = self.context
-        return simplejson.dumps(resource, cls=ResourceJSONEncoder)
+        return simplejson.dumps(
+            resource, cls=ResourceJSONEncoder,
+            media_type=EntryResource.JSON_TYPE)
 
 
 class WadlResourceAPI(RESTUtilityBase):
@@ -233,6 +293,11 @@ class WadlServiceRootResourceAPI(RESTUtilityBase):
         config = getUtility(IWebServiceConfiguration)
         return config.version_descriptions.get(self.service_version, None)
 
+    @property
+    def is_total_size_link_active(self):
+        config = getUtility(IWebServiceConfiguration)
+        return is_total_size_link_active(self.resource.request.version, config)
+
     @property
     def top_level_resources(self):
         """Return a list of dicts describing the top-level resources."""
@@ -249,7 +314,7 @@ class WadlServiceRootResourceAPI(RESTUtilityBase):
             resource_dicts.append({'name' : link_name,
                                    'path' : "$['%s']" % link_name,
                                    'resource' : resource})
-        return resource_dicts
+        return sorted(resource_dicts, key=operator.itemgetter('name'))
 
 
 class WadlResourceAdapterAPI(RESTUtilityBase):
@@ -377,11 +442,15 @@ class WadlEntryAdapterAPI(WadlResourceAdapterAPI):
         "The name of the description of a colleciton of this kind of object."
         return self.utility.entry_page_representation_id
 
+    @property
+    def publish_web_link(self):
+        return self.utility.publish_web_link
+
     @property
     def all_fields(self):
         "Return all schema fields for the object."
         return [field for name, field in
-                getFieldsInOrder(self.adapter.schema)]
+                sorted(getFields(self.adapter.schema).items())]
 
     @property
     def all_writable_fields(self):
@@ -455,8 +524,8 @@ class WadlFieldAPI(RESTUtilityBase):
         name = self.field.__name__
         if ICollectionField.providedBy(self.field):
             return name + '_collection_link'
-        elif (IObject.providedBy(self.field) or IBytes.providedBy(self.field)
-              or IReferenceChoice.providedBy(self.field)):
+        elif (IReference.providedBy(self.field)
+              or IBytes.providedBy(self.field)):
             return name + '_link'
         else:
             return name
@@ -491,8 +560,7 @@ class WadlFieldAPI(RESTUtilityBase):
     @property
     def is_represented_as_link(self):
         """Is this field represented as a link to another resource?"""
-        return (IObject.providedBy(self.field) or
-                IReferenceChoice.providedBy(self.field) or
+        return (IReference.providedBy(self.field) or
                 ICollectionField.providedBy(self.field) or
                 IBytes.providedBy(self.field) or
                 self.is_link)
@@ -525,16 +593,21 @@ class WadlFieldAPI(RESTUtilityBase):
         """Find an entry adapter for this field."""
         if ICollectionField.providedBy(self.field):
             schema = self.field.value_type.schema
-        elif (IObject.providedBy(self.field)
-              or IObjectLink.providedBy(self.field)
-              or IReferenceChoice.providedBy(self.field)):
+        elif (IReference.providedBy(self.field)
+              or IObjectLink.providedBy(self.field)):
             schema = self.field.schema
         else:
             raise TypeError("Field is not of a supported type.")
         assert schema is not IObject, (
             "Null schema provided for %s" % self.field.__name__)
-        return EntryAdapterUtility.forSchemaInterface(
-            schema, get_current_web_service_request())
+        try:
+            return EntryAdapterUtility.forSchemaInterface(
+                schema, get_current_web_service_request())
+        except UnknownEntryAdapter, e:
+            e.whence = (
+                'Encountered as a result of the entry interface %r, field %r.'
+                % (self.field.interface, self.field.getName()))
+            raise e
 
 
     @property
