@@ -5,6 +5,8 @@
 __metaclass__ = type
 __all__ = [
     'create_web_service_request',
+    'DummyAbsoluteURL',
+    'DummyRootResourceURL',
     'ExampleWebServiceTestCaller',
     'ExampleWebServicePublication',
     'FakeRequest',
@@ -12,6 +14,7 @@ __all__ = [
     'IGenericEntry',
     'IGenericCollection',
     'pprint_entry',
+    'simple_renderer',
     'WebServiceTestCase',
     'WebServiceTestConfiguration'
     'WebServiceTestPublication',
@@ -25,8 +28,9 @@ import traceback
 import simplejson
 import sys
 from types import ModuleType
-import urllib
 import unittest
+import urllib
+
 from urlparse import urljoin
 import wsgi_intercept
 
@@ -98,8 +102,7 @@ def create_web_service_request(path_info, method='GET', body=None,
 class FakeResponse:
     """Simple response wrapper object."""
     def __init__(self):
-        self.status = 599
-        self.headers = {}
+        self.reset()
 
     def setStatus(self, new_status):
         self.status = new_status
@@ -115,6 +118,21 @@ class FakeResponse:
         """Return the response status code."""
         return self.status
 
+    def reset(self):
+        """Set response values back to their initial state."""
+        self.status = 599
+        self.headers = {}
+        self.result = ''
+
+    def setResult(self, result):
+        if result is None:
+            result = ''
+
+        if not isinstance(result, basestring):
+            raise ValueError('only strings and None results are handled')
+
+        self.result = result
+
 
 class FakeRequest:
     """Simple request object for testing purpose."""
@@ -135,6 +153,8 @@ class FakeRequest:
         self.traversed_objects = []
         self.query_string_params = {}
         self.method = 'GET'
+        self.URL = 'http://api.example.org/'
+        self.headers = {}
 
 
     def getTraversalStack(self):
@@ -173,6 +193,8 @@ def pprint_entry(json_body):
 def pprint_collection(json_body):
     """Pretty-print a webservice collection JSON representation."""
     for key, value in sorted(json_body.items()):
+        if key == 'total_size_link':
+            continue
         if key != 'entries':
             print '%s: %r' % (key, value)
     print '---'
@@ -427,15 +449,15 @@ class IWebServiceTestRequest20(IWebServiceClientRequest):
     """A marker interface for requests to the '2.0' web service."""
 
 
-class WebServiceTestCase(CleanUp, unittest.TestCase):
-    """A test case for web service operations."""
+class TestCaseWithWebServiceFixtures(CleanUp, unittest.TestCase):
+    """A test case that needs to have some aspects of a web service set up.
 
-    testmodule_objects = []
+    If the test case is testing a real web service, use
+    WebServiceTestCase instead.
+    """
 
     def setUp(self):
-        """Set the component registry with the given model."""
-        super(WebServiceTestCase, self).setUp()
-
+        super(TestCaseWithWebServiceFixtures, self).setUp()
         # Register a simple configuration object.
         webservice_configuration = WebServiceTestConfiguration()
         sm = getGlobalSiteManager()
@@ -450,7 +472,52 @@ class WebServiceTestCase(CleanUp, unittest.TestCase):
         sm.registerUtility(
             IWebServiceTestRequest20, IWebServiceVersion, name='2.0')
 
+    def assertRaises(self, excClass, callableObj, *args, **kwargs):
+        """Fail unless an exception of class excClass is thrown
+           by callableObj when invoked with arguments args and keyword
+           arguments kwargs. If a different type of exception is
+           thrown, it will not be caught, and the test case will be
+           deemed to have suffered an error, exactly as for an
+           unexpected exception.
+
+           This is a copy of assertRaises from testtools.
+        """
+        try:
+            ret = callableObj(*args, **kwargs)
+        except excClass:
+            return sys.exc_info()[1]
+        else:
+            excName = self._formatTypes(excClass)
+            self.fail("%s not raised, %r returned instead." % (excName, ret))
+
+    def _formatTypes(self, classOrIterable):
+        """Format a class or a bunch of classes for display in an error.
+
+           This is a copy of _formatTypes from testtools.
+        """
+        className = getattr(classOrIterable, '__name__', None)
+        if className is None:
+            className = ', '.join(klass.__name__ for klass in classOrIterable)
+        return className
+
+    def fake_request(self, version):
+        request = FakeRequest(version=version)
+        alsoProvides(
+            request, getUtility(IWebServiceVersion, name=version))
+        return request
+
+
+class WebServiceTestCase(TestCaseWithWebServiceFixtures):
+    """A test case for web service operations."""
+
+    testmodule_objects = []
+
+    def setUp(self):
+        """Set the component registry with the given model."""
+        super(WebServiceTestCase, self).setUp()
+
         # Register a service root resource
+        sm = getGlobalSiteManager()
         service_root = ServiceRootResource()
         sm.registerUtility(service_root, IServiceRootResource)
 
@@ -526,15 +593,33 @@ class DummyRootResource:
         return {}
 
 
-class DummyRootResourceURL:
-    """A dummy IAbsoluteURL implementation for a DummyRootResource."""
+class DummyAbsoluteURL:
+    """Implements IAbsoluteURL for when you don't care what the URL is."""
     implements(IAbsoluteURL)
-    adapts(DummyRootResource, IWebServiceClientRequest)
-    def __init__(*args):
+    URL = 'http://dummyurl/'
+
+    def __init__(self, *args):
         pass
 
     def __str__(self):
-        return "http://dummy"
+        return self.URL
+
     __call__ = __str__
 
 
+class DummyRootResourceURL(DummyAbsoluteURL):
+    """A dummy IAbsoluteURL implementation for a DummyRootResource."""
+    adapts(DummyRootResource, IWebServiceClientRequest)
+
+    URL = 'http://dummyurl'
+
+
+def simple_renderer(value):
+     """Bold the original string and add a snowman.
+
+     To use this function, define an IHTMLFieldRenderer that returns it.
+
+     This is a good HTML field renderer to use in tests, because it
+     tests Unicode values and embedded HTML.
+     """
+     return u"\N{SNOWMAN} <b>%s</b>" % value
